//
//  HWDevicesView.m
//  OpenLive
//
//  Created by weshow on 2020/3/18.
//  Copyright © 2020 Agora.io. All rights reserved.
//

#import "HWDevicesView.h"
#import "HLMonitor.h"
@interface HWDevicesView ()<HwRtcEngineDelegate>
@property (weak) IBOutlet NSPopUpButton *videoDeviceButton;
@property (weak) IBOutlet NSPopUpButton *audioDeviceButton;
@property (weak) IBOutlet NSPopUpButton *recordingDeviceButton;
@property (weak) IBOutlet NSButton *closeButton;

@property (nonatomic, strong) NSString *videoDeviceID;
@property (nonatomic, strong) NSString *playbackDeviceID;
@property (nonatomic, strong) NSString *recordingDeviceID;

@property (nonatomic, strong) NSMutableArray *videoArray;
@property (nonatomic, strong) NSMutableArray *playbackArray;
@property (nonatomic, strong) NSMutableArray *recordingArray;

@property (nonatomic,strong) NSArray *profiles;


@end

@implementation HWDevicesView

- (void)drawRect:(NSRect)dirtyRect {
    [super drawRect:dirtyRect];
    self.wantsLayer = true;
    self.layer.backgroundColor = [NSColor whiteColor].CGColor;
    [self setNeedsDisplay:YES];
}

- (void)reloadVideoDevice:(NSArray *)videoDevices
{
    [self.videoArray removeAllObjects];
    [self.videoDeviceButton removeAllItems];
    [self.videoArray addObjectsFromArray:videoDevices];
    for (HwRtcDeviceInfo *deviceInfo in self.videoArray) {
        [self.videoDeviceButton addItemWithTitle:deviceInfo.deviceName];
    }
    videoDevices = nil;
}

- (void)reloadPlayBackDevice:(NSArray *)playBackDevices
{
    [self.playbackArray removeAllObjects];
    [self.audioDeviceButton removeAllItems];
    [self.playbackArray addObjectsFromArray:playBackDevices];
    for (HwRtcDeviceInfo *deviceInfo in self.playbackArray) {
        [self.audioDeviceButton addItemWithTitle:deviceInfo.deviceName];
    }
}

- (void)reloadRecordingDevice:(NSArray *)recordingDevices
{
    [self.recordingArray removeAllObjects];
    [self.recordingDeviceButton removeAllItems];
    [self.recordingArray addObjectsFromArray:recordingDevices];
    for (HwRtcDeviceInfo *deviceInfo in self.recordingArray) {
        [self.recordingDeviceButton addItemWithTitle:deviceInfo.deviceName];
    }
}

- (void)setCurrenVideoDevice:(NSString *)device
{
    NSString *deviceName = @"";
    for (HwRtcDeviceInfo *deviceInfo in self.videoArray) {
        NSString *deviceId = [NSString stringWithFormat:@"%@",deviceInfo.deviceId];
        if ([deviceId isEqualToString:device]) {
            deviceName = deviceInfo.deviceName;
            break;
        }
    }
    if (self.videoArray.count == 0) {
        self.videoDeviceID = @"";
    }
    if (deviceName.length != 0) {
        [self.videoDeviceButton selectItemWithTitle:deviceName];
        [self videoDeviceSelectAction:self.videoDeviceButton];
    }

}

- (void)setcurrenPlaybackDevice:(NSString *)device
{
    NSString *deviceName = @"";
    for (HwRtcDeviceInfo *deviceInfo in self.playbackArray) {
        if ([device isEqualToString:deviceInfo.deviceId]) {
            deviceName = deviceInfo.deviceName;
            break;
        }
    }
    if (deviceName.length != 0) {
        [self.audioDeviceButton selectItemWithTitle:deviceName];
        [self audioDeviceSelectAction:self.audioDeviceButton];
    }
}

- (void)setcurrenRecordingDevice:(NSString *)device
{
    NSString *deviceName = @"";
    for (HwRtcDeviceInfo *deviceInfo in self.recordingArray) {
        if ([device isEqualToString:deviceInfo.deviceId]) {
            deviceName = deviceInfo.deviceName;
            break;
        }
    }
    if (deviceName.length != 0) {
        [self.recordingDeviceButton selectItemWithTitle:deviceName];
        [self recordingDeviceSelectAction:self.recordingDeviceButton];
    }
}

- (IBAction)videoDeviceSelectAction:(NSPopUpButton *)sender {
    HwRtcDeviceInfo *info = [self.videoArray objectAtIndex:sender.indexOfSelectedItem];
    if (self.videoDeviceID != info.deviceId) {
        self.videoDeviceID = info.deviceId;
        if (self.delegate && [self.delegate respondsToSelector:@selector(devicesViewSelectVideoDeviceID:)]) {
            [self.delegate devicesViewSelectVideoDeviceID:self.videoDeviceID];
        }
        NSLog(@"setVideoDevice: deviceID%@",self.videoDeviceID);
    }
}

- (IBAction)audioDeviceSelectAction:(NSPopUpButton *)sender {
    HwRtcDeviceInfo *info = [self.playbackArray objectAtIndex:sender.indexOfSelectedItem];
    if (self.playbackDeviceID != info.deviceId) {
        self.playbackDeviceID = info.deviceId;
        if (self.delegate && [self.delegate respondsToSelector:@selector(devicesViewSelectPlaybackDeviceID:)]) {
            [self.delegate devicesViewSelectPlaybackDeviceID:self.playbackDeviceID];
        }
        NSLog(@"setPlaybackDevice: deviceID%@",self.videoDeviceID);
    }
}

- (IBAction)recordingDeviceSelectAction:(NSPopUpButton *)sender {
    HwRtcDeviceInfo *info = [self.recordingArray objectAtIndex:sender.indexOfSelectedItem];
    if (self.recordingDeviceID != info.deviceId) {
        self.recordingDeviceID = info.deviceId;
        if (self.delegate && [self.delegate respondsToSelector:@selector(devicesViewSelectRecordingDeviceID:)]) {
            [self.delegate devicesViewSelectRecordingDeviceID:self.recordingDeviceID];
        }
        NSLog(@"setRecordingDevice: deviceID%@",self.videoDeviceID);
    }
}


- (IBAction)changePlayBackVolume:(NSSlider *)sender {
    if (self.delegate && [self.delegate respondsToSelector:@selector(devicesViewChangePlayBackDeviceVolume:)]) {
        [self.delegate devicesViewChangePlayBackDeviceVolume:(int)sender.stringValue];
    }
}

- (IBAction)changeRecordingVolume:(NSSlider *)sender {
    if (self.delegate && [self.delegate respondsToSelector:@selector(devicesViewChangeRecordingDeviceVolume:)]) {
        [self.delegate devicesViewChangeRecordingDeviceVolume:(int)sender.stringValue];
    }
}

- (IBAction)closeView:(NSButton *)sender {
    self.hidden = YES;
}


#pragma mark - sett&&gett

- (NSArray *)profiles {
    if (!_profiles) {
        _profiles = @[@"SD",
                      @"HD",
                      @"FHD",
                      @"LD"];
    }
    return _profiles;
}

- (NSMutableArray *)videoArray
{
    if (!_videoArray) {
        _videoArray = [NSMutableArray array];
    }
    return _videoArray;
}

- (NSMutableArray *)playbackArray
{
    if (!_playbackArray) {
        _playbackArray = [NSMutableArray array];
    }
    return _playbackArray;
}

- (NSMutableArray *)recordingArray
{
    if (!_recordingArray) {
        _recordingArray = [NSMutableArray array];
    }
    return _recordingArray;
}
@end
