//
//  HWRoleListView.m
//  OpenLive
//
//  Created by weshow on 2020/3/10.
//  Copyright © 2020 Agora.io. All rights reserved.
//

#import "HWRoleListView.h"
#import "HWTableCellView.h"
#import "HLMonitor.h"
@interface HWRoleListView   ()<NSTableViewDelegate,NSTableViewDataSource,HWTableCellViewDelegate>
@property (nonatomic, strong) NSTableView *tableView;
@end

@implementation HWRoleListView

- (void)drawRect:(NSRect)dirtyRect {
    [super drawRect:dirtyRect];
    
    [self tableViewSetting];
}

- (void)setViewsArray:(NSMutableArray *)viewsArray
{
    _viewsArray = viewsArray;
    [self.tableView reloadData];
}

- (NSTableView *)tableView
{
    if(!_tableView){
        _tableView = [[NSTableView alloc] initWithFrame:CGRectMake(0 , 0, 200, 300)];
    }
    return _tableView;
}

- (void)tableViewSetting{
  
    //第二列
    NSTableColumn * column = [[NSTableColumn alloc] initWithIdentifier:@"rolesList"];
    column.title = @"rolesList";
    [column setWidth:200];
    [self.tableView addTableColumn:column];

    self.tableView.headerView.frame = NSZeroRect;
    self.tableView.focusRingType = NSFocusRingTypeNone;//tableview获得焦点时的风格
    
    self.tableView.selectionHighlightStyle = NSTableViewSelectionHighlightStyleRegular;//行高亮的风格
    
    self.tableView.usesAlternatingRowBackgroundColors = NO; //背景颜色的交替，一行白色，一行灰色。设置后，原来设置的 backgroundColor 就无效了。
    self.tableView.selectionHighlightStyle = NSTableViewSelectionHighlightStyleNone;
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    
    self.tableView.gridColor = [NSColor magentaColor];
    
    //实现tableview的滚动效果
    
    NSScrollView *tableContainerView = [[NSScrollView alloc] initWithFrame:CGRectMake(0, 0, 200, 300)];
    
    [tableContainerView setDocumentView:self.tableView];
    
    [tableContainerView setDrawsBackground:NO];//不画背景（背景默认画成白色）
    
    [tableContainerView setHasVerticalScroller:YES];//有垂直滚动条
    
    //[_tableContainer setHasHorizontalScroller:YES];  //有水平滚动条
    
    tableContainerView.autohidesScrollers = YES;//自动隐藏滚动条（滚动的时候出现）
    
    
    [self addSubview:tableContainerView];
}
#pragma mark - NSTableViewDelegate,NSTableViewDataSource

//返回行数
-(NSInteger)numberOfRowsInTableView:(NSTableView *)tableView{
    return self.viewsArray.count;
}


-(NSView *)tableView:(NSTableView *)tableView viewForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row{
    
   HWTableCellView *cell = [tableView makeViewWithIdentifier:tableColumn.identifier owner:self];
    if (!cell) {
        cell = [[HWTableCellView alloc]init];
        cell.identifier = tableColumn.identifier;
    }
    HWVideoSession *videoSession = [self.viewsArray objectAtIndex:row];
    cell.usernameTextField.stringValue = videoSession.userName;
    cell.isMuteLocalVideo = !videoSession.isPlaying;
    cell.index = (int)row;
    cell.delegate = self;
    return cell;
}

#pragma mark - 行高
-(CGFloat)tableView:(NSTableView *)tableView heightOfRow:(NSInteger)row{
    return 24;
}

#pragma mark - 是否可以选中单元格


#pragma mark -m HWTableCellViewDelegate
- (void)tableCellViewSelectWithIndex:(int)index
{
    HWVideoSession *videoSession = [self.viewsArray objectAtIndex:index];

    if (4<=self.showingCount) {
        if(!videoSession.isPlaying){
            [HLMonitor showMessageWithStr:@"show view more than limit!"];
            NSLog(@"show view more than limit!");
            return;
        }
    }
    videoSession.isPlaying = !videoSession.isPlaying;
    if (!videoSession.isPlaying) {
        [self.reusePool removeVideoView:videoSession.videoView];
        videoSession.videoView = nil;
        self.showingCount--;
        
    } else {
        videoSession.videoView = [self.reusePool dequeueReusableView];
        if (!videoSession.videoView) {
            videoSession.videoView = [[HwRtcVideoView alloc] initWithFrame:NSRectFromCGRect(CGRectMake(0, 0, 90, 160))];
            [self.reusePool addVideoView:videoSession.videoView];
            NSLog(@"===重用池出错了===");
        }
        [videoSession.videoView setUserName:videoSession.userName];
        self.showingCount++;
    }
    BOOL result = YES;
    if (videoSession.isSubStream) {
        result = [_rtcEngine setupRemoteSubStreamView:videoSession.canvas viewMode:HwRtcVideoDisplayModeCrop streamType:HwRtcStreamTypeHD];
    } else
    {
        
        result = [_rtcEngine setupRemoteView:videoSession.canvas
                                         viewMode:HwRtcVideoDisplayModeCrop
                                       streamType:HwRtcStreamTypeHD];
    }
    NSLog(@"|||sdk==%@ ,%@,窗口对象==%@,result = %d,username =%@,窗口hidden=%d,窗口父窗口==%@,%@|||",
          _rtcEngine,
          (videoSession.isPlaying?@"设置窗口":@"取消窗口"),
          videoSession.canvas.view,
          result,
          videoSession.userName,
          videoSession.canvas.view.hidden,
          videoSession.canvas.view.superview,
          videoSession.canvas.view.class);
    [self.tableView reloadData];
    if (self.selectBlock) {
        self.selectBlock(videoSession);
    }
}


@end
