//
//  LiveRoomViewController.m
//  OpenLive
//
//  Created by yangmoumou on 2018/2/8.
//  Copyright © 2018年 yangmoumou. All rights reserved.
//

#import "LiveRoomViewController.h"
#import "HWRoleListView.h"
#import "HWDevicesView.h"
#import "HLMonitor.h"
#import "HWVideoSession.h"
#import "HWReusePool.h"

#define  MAXSCREENCOUNT 4

@interface LiveRoomViewController ()<HwRtcEngineDelegate,HWDevicesViewDelegate>
@property (weak) IBOutlet NSView *remoteContainerView;
@property (weak) IBOutlet NSButton *muteAudioButton;
@property (weak) IBOutlet NSButton *muteLocalVideoButton;
@property (weak) IBOutlet NSButton *muteLocalAudioButton;
@property (weak) IBOutlet NSButton *roleList;
@property (weak) IBOutlet NSButton *changeRoleTypeButton;
@property (weak) IBOutlet NSButton *uploadLogButton;
@property (weak) IBOutlet NSButton *realTimeInfoButton;

@property (weak) IBOutlet NSTextField *roomNameLabel;
@property (strong) IBOutlet HWRoleListView *roleListView;
@property (weak) IBOutlet NSButton *devicesButton;
@property (nonatomic, strong) HWDevicesView *devicesView;

@property (nonatomic, assign) BOOL isBroadcaster;
@property (nonatomic, assign) BOOL isMuted;

@property (strong, nonatomic) HWVideoSession *localVideoSession;
@property (strong, nonatomic) HWVideoSession *subStreamVideoSession;
@property (nonatomic, strong) NSMutableArray *viewsArray;
@property (nonatomic, strong) NSMutableArray *showViewsArray;

@property (nonatomic, assign) BOOL isLocalVideo;
@property (nonatomic, assign) BOOL isLocalAudio;
@property (nonatomic, assign) BOOL isSemoteVideo;

///重用池
@property (nonatomic, strong) HWReusePool *reusePool;
@property (nonatomic, assign) HwRtcStreamType streamType;

@property (nonatomic, strong) NSString *defaultStreamUserid;
@property (nonatomic, assign) HwRtcStreamType defaultStreamType;
@property (nonatomic, assign) BOOL isJoinRoomSuccess;
@end

@implementation LiveRoomViewController

- (HWReusePool *)reusePool
{
    if (!_reusePool) {
        _reusePool = [[HWReusePool alloc] init];
    }
    return _reusePool;
}

- (NSMutableArray *)viewsArray
{
    if (!_viewsArray) {
        _viewsArray = [NSMutableArray array];
    }
    return _viewsArray;
}

- (NSMutableArray *)showViewsArray
{
    if (!_showViewsArray) {
        _showViewsArray = [NSMutableArray array];
    }
    return _showViewsArray;
}

- (HWDevicesView *)devicesView
{
    if (!_devicesView) {
        NSNib *xib = [[NSNib alloc] initWithNibNamed:@"HWDevicesView" bundle:nil];
        NSArray *viewsArray = [[NSArray alloc] init];
        [xib instantiateWithOwner:nil topLevelObjects:&viewsArray];
        for (int i = 0; i < viewsArray.count; i++) {
            if ([viewsArray[i] isKindOfClass:[HWDevicesView class]]) {
                _devicesView = (HWDevicesView *)viewsArray[i];
                break;
            }
        }
        [_devicesView setFrameOrigin:NSPointFromCGPoint(CGPointMake(60, 125))];
        [self.view addSubview:_devicesView];
        _devicesView.hidden = YES;
        _devicesView.delegate = self;
    }
    return _devicesView;
}

- (void)dealloc
{
    NSLog(@"%@ dealloc",self.className);
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.rtcEngine.delegate         = self;
    self.roomNameLabel.stringValue  = self.roomName;
    _isLocalAudio                   = YES;
    _isLocalVideo                   = YES;
    _isSemoteVideo                  = YES;
    _isJoinRoomSuccess              = NO;
    self.streamType                 = HwRtcStreamTypeFHD;
    if (self.roleType == HwRtcRoleJoiner) {
        self.changeRoleTypeButton.image = [NSImage imageNamed:@"btn_join_cancel"];
    } else if (self.roleType == HwRtcRolePlayer)
    {
        self.changeRoleTypeButton.image = [NSImage imageNamed:@"btn_join"];
    } else
    {
        self.changeRoleTypeButton.image = [NSImage imageNamed:@"btn_join"];
        self.changeRoleTypeButton.enabled = NO;
    }
    [self insertNotificationCenter];
    [self joinChannel];
    
    self.roleListView.rtcEngine = self.rtcEngine;
    self.roleListView.reusePool = self.reusePool;
    
    __weak typeof(self) weakSelf    = self;
    self.roleListView.selectBlock   = ^(HWVideoSession * _Nullable videoSession) {
        __strong typeof(weakSelf) strongSelf = weakSelf;
        if (videoSession.isPlaying) {
            [strongSelf.showViewsArray addObject:videoSession];
        } else
        {
            [strongSelf.showViewsArray removeObject:videoSession];
        }
        [strongSelf reloadViewWithDataArray:strongSelf.showViewsArray];
    };
    [self configurationDevice];
    [self getDeviceStatus];
    NSLog(@"viewDidLoad");
}

- (void)configurationDevice
{
    if (self.roleType == HwRtcRolePublisher) {
        self.roleList.enabled        = NO;
        self.muteAudioButton.enabled = NO;
    }else if (self.roleType == HwRtcRolePlayer) {
        self.muteLocalVideoButton.enabled = NO;
        self.muteLocalAudioButton.enabled = NO;
    }else
    {
        self.muteLocalVideoButton.enabled = YES;
        self.muteLocalAudioButton.enabled = YES;
    }
    
}

- (void)getDeviceStatus
{
    [self.rtcEngine setplaybackDeviceMute:NO];
    [self.rtcEngine setRecordingDeviceMute:NO];
}

- (void)joinChannel
{
    HwRtcUserInfo *userInfo = [[HwRtcUserInfo alloc]init];
    userInfo.role       = self.roleType;
    userInfo.userId     = self.userName;
    userInfo.userName   = self.userName;
    int iRet =  [_rtcEngine joinRoom:userInfo roomId:self.roomName mediaType:HwRtcMediaTypeVideo];
    if (iRet!=0) {
        [self rtcEngineLeaveRoomCallback];
        NSLog(@"加入房间失败");
    }
    
    if (self.roleType != HwRtcRolePlayer) {
        _localVideoSession = [[HWVideoSession alloc] initWithUserID:self.userName];
        HwRtcVideoView *videoView = [[HwRtcVideoView alloc] initWithFrame:self.view.bounds];
        _localVideoSession.videoView = videoView;
        _localVideoSession.userName  = self.userName;
        [_rtcEngine setupLocalVideo:_localVideoSession.canvas viewMode:HwRtcVideoDisplayModeCrop];
        [self.remoteContainerView addSubview:videoView];
        [self.showViewsArray addObject:_localVideoSession];
        [self reloadViewWithDataArray:self.showViewsArray];
    }
     NSLog(@"joinChannel");
}


- (void)reloadViewWithDataArray:(NSMutableArray *)dataArray
{
    CGFloat width = 0;
    CGFloat height = 0;
    CGFloat offset = 0;
    width = _screenWidth;
    height = _screenHeight;
    NSLog(@"|||更新试图的数组内容:%@|||",dataArray);
    NSLog(@"|||viewsArray == %@|||",self.viewsArray);
    if(dataArray.count==1){
        HWVideoSession *videoSession = [dataArray objectAtIndex:0];
        videoSession.videoView.frame = CGRectMake(0, offset, width, height);
        [videoSession.videoView setUserName:videoSession.userName];
    }else if(dataArray.count==2){
        HWVideoSession *videoSession = [dataArray objectAtIndex:0];
        videoSession.videoView.frame = CGRectMake(0, offset+height/2, width, height/2);
        [videoSession.videoView setUserName:videoSession.userName];
        
        HWVideoSession *videoSession1 = [dataArray objectAtIndex:1];
        videoSession1.videoView.frame = CGRectMake(0, offset, width, height/2);
        [videoSession1.videoView setUserName:videoSession1.userName];
    }else if(dataArray.count==3){
        HWVideoSession *videoSession = [dataArray objectAtIndex:0];
        videoSession.videoView.frame = CGRectMake(0, offset+height/2, width, height/2);
        [videoSession.videoView setUserName:videoSession.userName];
        
        HWVideoSession *videoSession1 = [dataArray objectAtIndex:1];
        videoSession1.videoView.frame = CGRectMake(0, offset, width/2, height/2);
        [videoSession1.videoView setUserName:videoSession1.userName];
        
        HWVideoSession *videoSession2 = [dataArray objectAtIndex:2];
        videoSession2.videoView.frame = CGRectMake(width/2, offset,  width/2, height/2);
        [videoSession2.videoView setUserName:videoSession2.userName];
    }else if(dataArray.count==4){
        HWVideoSession *videoSession = [dataArray objectAtIndex:0];
        videoSession.videoView.frame = CGRectMake(0, offset+height/2, width/2, height/2);
        [videoSession.videoView setUserName:videoSession.userName];
        
        HWVideoSession *videoSession1 = [dataArray objectAtIndex:1];
        videoSession1.videoView.frame = CGRectMake(width/2, offset+height/2, width/2, height/2);
        [videoSession1.videoView setUserName:videoSession1.userName];
        
        HWVideoSession *videoSession2 = [dataArray objectAtIndex:2];
        videoSession2.videoView.frame = CGRectMake(0, offset, width/2, height/2);
        [videoSession2.videoView setUserName:videoSession2.userName];
        
        HWVideoSession *videoSession3 = [dataArray objectAtIndex:3];
        videoSession3.videoView.frame = CGRectMake(width/2, offset,  width/2, height/2);
        [videoSession3.videoView setUserName:videoSession3.userName];
    }
}

#pragma mark -Notification

- (void)insertNotificationCenter
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(screenResize) name:NSWindowDidResizeNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowWillEnterFullScreen:) name:NSWindowWillEnterFullScreenNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowWillExitFullScreen:) name:NSWindowWillExitFullScreenNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidEnterFullScreen:) name:NSWindowDidEnterFullScreenNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidExitFullScreen:) name:NSWindowDidExitFullScreenNotification object:nil];
}

// 窗口拉伸
- (void)screenResize{
    if (self.view.window.frame.size.width == 0 &&
        self.view.window.frame.size.height == 0) {
        return;
    }
    _screenWidth = self.view.window.frame.size.width;
    _screenHeight = self.view.window.frame.size.height;
    [self reloadViewWithDataArray:self.showViewsArray];
    NSLog(@"screenResize");
}

- (void)windowWillEnterFullScreen:(NSNotificationCenter *)notification
{
    _screenWidth = [NSScreen mainScreen].frame.size.width;
    _screenHeight = [NSScreen mainScreen].frame.size.height;
    [self reloadViewWithDataArray:self.showViewsArray];
    NSLog(@"windowWillEnterFullScreen");
}

- (void)windowWillExitFullScreen:(NSNotificationCenter *)notification
{
    _screenWidth = 480;
    _screenHeight = 640;
    [self reloadViewWithDataArray:self.showViewsArray];
    NSLog(@"windowWillExitFullScreen");
}

- (void)windowDidEnterFullScreen:(NSNotificationCenter *)notification
{
    [self.devicesView setFrameOrigin:NSPointFromCGPoint(CGPointMake((_screenWidth-360)/2, 300))];
    NSLog(@"windowDidEnterFullScreen");
}

- (void)windowDidExitFullScreen:(NSNotificationCenter *)notification
{
    [_devicesView setFrameOrigin:NSPointFromCGPoint(CGPointMake(60, 125))];
    NSLog(@"muteLocalVideoAction");
}

#pragma mark - XIB
//切换角色
- (IBAction)changeRoleTypeAction:(NSButton *)sender {

    if (self.roleType == HwRtcRoleJoiner) {
        self.roleType =  HwRtcRolePlayer;
        sender.image = [NSImage imageNamed:@"btn_join"];
    } else
    {
        self.roleType = HwRtcRoleJoiner;
        sender.image = [NSImage imageNamed:@"btn_join_cancel"];
    }
    [self.rtcEngine setUserRole:self.roleType];
    sender.enabled = NO;
    NSLog(@"changeRoleTypeAction");
    
}

// 开关本地摄像头
- (IBAction)muteLocalVideoAction:(NSButton *)sender {
    _isLocalVideo = !_isLocalVideo;
    if (!self.isLocalVideo) {
        sender.image = [NSImage imageNamed:@"video_off"];
    } else {
        sender.image = [NSImage imageNamed:@"video_on"];
    }
    [self.rtcEngine muteLocalVideo:!_isLocalVideo];
    NSLog(@"muteLocalVideoAction");
}

// 开关麦克风
- (IBAction)muteLocalAudioAction:(NSButton *)sender {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(buttonClickedAction:) object:sender];
    [self performSelector:@selector(buttonClickedAction:) withObject:sender afterDelay:0.5];
}

- (void)buttonClickedAction:(NSButton *)sender
{
    _isLocalAudio = !_isLocalAudio;
    if (self.isLocalAudio) {
        sender.image = [NSImage imageNamed:@"mic_on"];
    } else {
        sender.image = [NSImage imageNamed:@"mic_off"];
    }
    [self.rtcEngine adjustRecordingVolume:_isLocalAudio?10:0];
    NSLog(@"muteLocalAudioAction");
}

// 开关扬声器
- (IBAction)doMuteClicked:(NSButton *)sender {
    self.isMuted = !self.isMuted;
    if (!self.isMuted) {
        sender.image = [NSImage imageNamed:@"speaker_on"];
    } else {
        sender.image = [NSImage imageNamed:@"speaker_off"];
    }
    [self.rtcEngine adjustPlaybackVolume:_isMuted?0:10];

}

/// 退出房间
- (IBAction)doLeaveClicked:(NSButton *)sender {
    if (!_isJoinRoomSuccess) {
        return;
    }
    [_rtcEngine leaveRoom];
    NSLog(@"doLeaveClicked");
}

/// 远程列表
- (IBAction)openRoleListAction:(NSButton *)sender {
    
    if (!self.devicesView.hidden) {
        self.devicesView.hidden = YES;
        
    }
    self.roleListView.showingCount = self.showViewsArray.count;
    self.roleListView.hidden = !self.roleListView.hidden;
    NSLog(@"openRoleListAction");
}


- (IBAction)openDeviceManageAction:(NSButton *)sender {
    if (!self.roleListView.hidden) {
        self.roleListView.hidden = YES;
    }
    if (_screenWidth>480) {
        [self.devicesView setFrameOrigin:NSPointFromCGPoint(CGPointMake((_screenWidth-360)/2, 150))];
    }else
    {
        [_devicesView setFrameOrigin:NSPointFromCGPoint(CGPointMake(60, 125))];
    }
    
    self.devicesView.hidden = !self.devicesView.hidden;
    if (!self.devicesView.hidden && !self.devicesView.isFirst) {
        self.devicesView.isFirst = YES;
        [self.devicesView reloadVideoDevice:[self.rtcEngine getVideoDevices]];
        [self.devicesView reloadPlayBackDevice:[self.rtcEngine getPlaybackDevices]];
        [self.devicesView reloadRecordingDevice:[self.rtcEngine getRecordingDevices]];
        NSString *currenVideoDevice = [self.rtcEngine getCurrentVideoDevice];
        [self.devicesView setCurrenVideoDevice:currenVideoDevice];
        NSString *currenPlaybackDevice = [self.rtcEngine getCurrentPlaybackDevice];
        [self.devicesView setcurrenPlaybackDevice:currenPlaybackDevice];
        NSString *currenRecordingDevice = [self.rtcEngine getCurrentRecordingDevice];
        [self.devicesView setcurrenRecordingDevice:currenRecordingDevice];
    }
    NSLog(@"openDeviceManageAction");
}

- (IBAction)uploadLogButtonAction:(NSButton *)sender {
    [self.rtcEngine logUpload];
    self.uploadLogButton.image = nil;
     NSLog(@"uploadLogButtonAction");
}

#pragma mark - HwRtcEngineDelegate

-(void)rtcEngineJoinSuccessCallback:(NSString*)roomId userId:(NSString*)userId{
    _isJoinRoomSuccess = YES;
    NSLog(@"zhanganl HwRtcEngineJoinSuccessCallback %@",userId);
}

-(void)rtcEngineLeaveRoomCallback{
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.delegate liveVCNeedClose:self];
        [[NSNotificationCenter defaultCenter] removeObserver:self];
        NSLog(@"rtcEngineLeaveRoomCallback");
    });
}

-(void)rtcEngineUserJoinedCallback:(NSString*)roomId userId:(NSString*)userId nickName:(NSString*)nickName{
    /// 如果是默认流,就不用重复加入了 (defaultStreamUserid)
    if([userId isEqualToString:self.userName] || self.roleType == HwRtcRolePublisher || [userId isEqualToString:self.defaultStreamUserid]){
        return;
    }
    dispatch_async(dispatch_get_main_queue(),
                   ^{
        NSLog(@"加入房间 roomId = %@,userId = %@, nickName = %@",roomId,userId,nickName);
        HWVideoSession *videoSession = [[HWVideoSession alloc] initWithUserID:userId];
        videoSession.isPlaying = NO;
        videoSession.userName  = userId;
        [self.viewsArray addObject:videoSession];
        if (self.showViewsArray.count < 4) {
            HwRtcVideoView *videoView = [self.reusePool dequeueReusableView];
            if (!videoView) {
                videoView = [[HwRtcVideoView alloc] initWithFrame:NSRectFromCGRect(CGRectMake(0, 0, 90, 160))];
                [self.reusePool addVideoView:videoView];
            }
            videoSession.videoView = videoView;
            videoSession.isPlaying = YES;
            [self.remoteContainerView addSubview:videoView];
            if (self.showViewsArray.count > 1 && self.streamType == HwRtcStreamTypeFHD) {
                self.streamType = HwRtcStreamTypeHD;
            }
            BOOL result = [self.rtcEngine setupRemoteView:videoSession.canvas
                                   viewMode:HwRtcVideoDisplayModeCrop
                                 streamType:self.streamType];
            NSLog(@"|||rtcEngineUserJoinedCallback,SDK对象==%@,窗口对象==%@,result = %d,username =%@,窗口hidden=%d,窗口父窗口==%@,%@|||",
                  _rtcEngine,
                  videoSession.canvas.view,
                  result,
                  videoSession.userName,
                  videoSession.canvas.view.hidden,
                  videoSession.canvas.view.superview,
                  videoSession.canvas.view.class);
            [self.showViewsArray addObject:videoSession];
            [self reloadViewWithDataArray:self.showViewsArray];
            
        }
        if (self.roleListView) {
            self.roleListView.roleType = self.roleType;
            self.roleListView.showingCount = self.showViewsArray.count;
            self.roleListView.viewsArray = self.viewsArray;
        }
    });

}

- (void)rtcEngineUserOfflineCallback:(NSString *)roomId userId:(NSString *)userId reason:(NSInteger)reason
{
    if(self.roleType == HwRtcRolePublisher){
        return;
    }
    dispatch_async(dispatch_get_main_queue(),
    ^{
        NSLog(@"rtcEngineUserOfflineCallback roomId:%@,userId:%@,reason:%ld",roomId,userId,(long)reason);
        HWVideoSession *offlineVideoSession;
        for (HWVideoSession *videoSession in self.viewsArray) {
            if ([userId isEqualToString:videoSession.canvas.uid] && !videoSession.isSubStream) {
                offlineVideoSession = videoSession;
                break;
            }
        }
        if (offlineVideoSession) {
            [self.viewsArray removeObject:offlineVideoSession];
            [self.reusePool removeVideoView:offlineVideoSession.videoView];
            BOOL isLineShowView = NO;
            if (self.showViewsArray.count > 0) {
                for (HWVideoSession *videoSession in self.showViewsArray) {
                    if ([videoSession.canvas.uid isEqualToString:userId] && !videoSession.isSubStream) {
                        isLineShowView = YES;
                        break;
                    }
                }
            }
            if (isLineShowView) {
                [self.showViewsArray removeObject:offlineVideoSession];
                [self reloadViewWithDataArray:self.showViewsArray];
            }
            
        }
        if (self.roleListView) {
            self.roleListView.showingCount = self.showViewsArray.count;
            self.roleListView.viewsArray = self.viewsArray;
        }
           
        
    });
}

- (void)rtcEngineUserRoleChangeWithNewRole:(HwRtcRole)newRole oldRole:(HwRtcRole)oldRole
{
    dispatch_async(dispatch_get_main_queue(), ^{
        
        if (self.roleType == HwRtcRolePlayer) {
            [self.showViewsArray removeObject:self.localVideoSession];
            self.localVideoSession.isPlaying = NO;
            [self.reusePool removeVideoView:self.localVideoSession.videoView];
            self.localVideoSession.videoView = nil;
            
            NSLog(@"|||切换角色为player");
        } else
        {
            ///如果选看四个了.就抛弃最后一个选看,把本地的显示出来
            if (self.showViewsArray.count == 4) {
                HWVideoSession *lastVideoSession = [self.showViewsArray lastObject];
                [self.reusePool removeVideoView:lastVideoSession.videoView];
                lastVideoSession.videoView = nil;
                lastVideoSession.isPlaying = NO;
                [self.rtcEngine setupRemoteView:lastVideoSession.canvas
                                       viewMode:HwRtcVideoDisplayModeCrop
                                     streamType:self.streamType];
                [self.showViewsArray removeObject:lastVideoSession];
                self.roleListView.viewsArray = self.viewsArray; ///刷新试图
            }
            
            if (!self.localVideoSession) {
                self.localVideoSession = [[HWVideoSession alloc] initWithUserID:self.userName];
                self.localVideoSession.userName  = self.userName;
            }
            HwRtcVideoView *videoView = [self.reusePool dequeueReusableView];
            if (!videoView) {
                videoView = [[HwRtcVideoView alloc] initWithFrame:self.view.bounds];
                [self.reusePool addVideoView:videoView];
                [self.remoteContainerView addSubview:videoView];
            }
            self.localVideoSession.videoView = videoView;
            
            BOOL result = [self.rtcEngine setupLocalVideo:self.localVideoSession.canvas
                                             viewMode:HwRtcVideoDisplayModeCrop];
            [self.showViewsArray insertObject:self.localVideoSession atIndex:0];
            NSLog(@"|||切换角色wei joiner,SDK对象==%@,窗口对象==%@,result = %d,username =%@,窗口hidden=%d,窗口父窗口==%@,%@|||",
                  self.rtcEngine,
                  self.localVideoSession.canvas.view,
                  result,
                  self.localVideoSession.userName,
                  self.localVideoSession.canvas.view.hidden,
                  self.localVideoSession.canvas.view.superview,
                  self.localVideoSession.canvas.view.class);
        }
        [self reloadViewWithDataArray:self.showViewsArray];
        [self configurationDevice];
        self.changeRoleTypeButton.enabled = YES;
    });
    
}

- (void)rtcEngineConnStateChangeCallback:(HwRtcConnStateType)state reason:(HwRtcConnChangeReason)reason description:(NSString *)description
{
    dispatch_async(dispatch_get_main_queue(), ^{
        if (state == HwRtcConnStateTypeReconnecting) {
            [HLMonitor showMessageWithStr:description];
        }
    });
}

-(void)rtcEngineErrorCallback:(int)errorCode errorMsg:(NSString*)errorMsg;
{
    if (self.roleType == HwRtcRolePublisher) {
        return;
    }
    dispatch_async(dispatch_get_main_queue(), ^{
        //切换失败
        if (errorCode == HwrtcErrorCodeUserRoleChangeFail) {
            if (self.roleType == HwRtcRoleJoiner) {
                self.roleType =  HwRtcRolePlayer;
                self.changeRoleTypeButton.image = [NSImage imageNamed:@"btn_join"];
            } else
            {
                self.roleType = HwRtcRoleJoiner;
                self.changeRoleTypeButton.image = [NSImage imageNamed:@"btn_join_cancel"];
            }
            self.changeRoleTypeButton.enabled = YES;
            NSLog(@"切换角色失败");
        }
        [HLMonitor showMessageWithStr:[NSString stringWithFormat:@"errorCode = %d \nerrorMsg = %@",errorCode,errorMsg]];
    });
}

- (void)rtcEngineSubStreamAvailableWitRoomId:(NSString *)roomId userId:(NSString *)userId isAvailable:(BOOL)isAvailable
{
    dispatch_async(dispatch_get_main_queue(), ^{
        if (isAvailable) {
            ///如果选看四个了.就抛弃最后一个选看,把本地的显示出来
            if (self.showViewsArray.count == 4) {
                HWVideoSession *lastVideoSession = [self.showViewsArray lastObject];
                [self.reusePool removeVideoView:lastVideoSession.videoView];
                lastVideoSession.videoView = nil;
                lastVideoSession.isPlaying = NO;
                [self.rtcEngine setupRemoteView:lastVideoSession.canvas
                                       viewMode:HwRtcVideoDisplayModePad
                                     streamType:self.streamType];
                [self.showViewsArray removeObject:lastVideoSession];
                self.roleListView.viewsArray = self.viewsArray; ///刷新试图
            }
            if (!self.subStreamVideoSession) {
                self.subStreamVideoSession = [[HWVideoSession alloc] initWithUserID:userId];
                self.subStreamVideoSession.isSubStream = YES;
            }
            self.subStreamVideoSession.isPlaying = YES;
            self.subStreamVideoSession.userName  = [NSString stringWithFormat:@"辅流-%@",userId];
            HwRtcVideoView *videoView = [self.reusePool dequeueReusableView];
            if (!videoView) {
                videoView = [[HwRtcVideoView alloc] initWithFrame:self.view.bounds];
                [self.reusePool addVideoView:videoView];
                [self.remoteContainerView addSubview:videoView];
            }
            [videoView setUserName:self.subStreamVideoSession.userName];
            self.subStreamVideoSession.videoView = videoView;
            int result = [self.rtcEngine setupRemoteSubStreamView:self.subStreamVideoSession.canvas viewMode:HwRtcVideoDisplayModePad streamType:self.streamType];
            NSLog(@"辅流result%d",result);
            [self.showViewsArray insertObject:self.subStreamVideoSession atIndex:0];
            [self.viewsArray insertObject:self.subStreamVideoSession atIndex:0];
            
        }else {
            ///无需再手动取消辅流
            self.subStreamVideoSession.isPlaying = NO;
            [self.reusePool removeVideoView:self.subStreamVideoSession.videoView];
            [self.showViewsArray removeObject:self.subStreamVideoSession];
            [self.viewsArray removeObject:self.subStreamVideoSession];
            NSLog(@"取消辅流result");
        }
        self.roleListView.viewsArray = self.viewsArray;
        [self reloadViewWithDataArray:self.showViewsArray];
    });
}

//默认流处理
- (void)rtcEngineStreamAvailableWithRoomid:(NSString *)roomid userid:(NSString *)userid streamType:(HwRtcStreamType)streamType
{
    NSLog(@"rtcEngineStreamAvailableWithRoomid");
}

- (void)rtcEngineJoinRoomFailure
{
    NSLog(@"rtcEngineJoinRoomFailure");
    dispatch_async(dispatch_get_main_queue(), ^{
        [self rtcEngineLeaveRoomCallback];
        [HLMonitor showMessageWithStr:[NSString stringWithFormat:@"加入房间失败"]];
    });
}

- (void)rtcEngineDeviceStateChangedCallback:(NSString *)deviceId deviceType:(HwRtcDeviceType)deviceType deviceState:(HwRtcDeviceState)deviceState
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.devicesView reloadVideoDevice:[self.rtcEngine getVideoDevices]];
        [self.devicesView reloadPlayBackDevice:[self.rtcEngine getPlaybackDevices]];
        [self.devicesView reloadRecordingDevice:[self.rtcEngine getRecordingDevices]];
        NSString *currenVideoDevice = [self.rtcEngine getCurrentVideoDevice];
        [self.devicesView setCurrenVideoDevice:currenVideoDevice];
        NSString *currenPlaybackDevice = [self.rtcEngine getCurrentPlaybackDevice];
        [self.devicesView setcurrenPlaybackDevice:currenPlaybackDevice];
        NSString *currenRecordingDevice = [self.rtcEngine getCurrentRecordingDevice];
        [self.devicesView setcurrenRecordingDevice:currenRecordingDevice];
        NSLog(@"%@,%ld,%ld",
              deviceId,
              (long)deviceType,
              (long)deviceState);
    });
}

- (void)rtcEngineLogUploadProgressCallback:(int)progress
{
    NSLog(@"rtcEngineLogUploadProgressCallback");
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.uploadLogButton setTitle:[NSString stringWithFormat:@"%d",progress]];
    });
}

- (void)rtcEngineLogUploadResultCallback:(int)result
{
     NSLog(@"rtcEngineLogUploadResultCallback");
    dispatch_async(dispatch_get_main_queue(), ^{
        if (result==0) {
            [HLMonitor showMessageWithStr:@"上传成功"];
        } else
        {
            [HLMonitor showMessageWithStr:@"上传失败"];
        }
        self.uploadLogButton.image = [NSImage imageNamed:@"uploadLog"];
    });
}


#pragma mark - HWDeviceViewDelegate
- (void)devicesViewSelectVideoDeviceID:(NSString *)videoDeviceID
{
     NSLog(@"devicesViewSelectVideoDeviceID");
    [self.rtcEngine setVideoDevice:videoDeviceID];
}

- (void)devicesViewSelectPlaybackDeviceID:(NSString *)playbackDeviceID
{
     NSLog(@"devicesViewSelectPlaybackDeviceID");
    [self.rtcEngine setPlaybackDevice:playbackDeviceID];
}

- (void)devicesViewSelectRecordingDeviceID:(NSString *)recordingDeviceID
{
     NSLog(@"devicesViewSelectRecordingDeviceID");
    [self.rtcEngine setRecordingDevice:recordingDeviceID];
}

- (void)devicesViewChangePlayBackDeviceVolume:(int)volume
{
     NSLog(@"devicesViewChangePlayBackDeviceVolume");
    [self.rtcEngine setPlaybackDevicesVolume:volume];
}

- (void)devicesViewChangeRecordingDeviceVolume:(int)volume
{
     NSLog(@"devicesViewChangeRecordingDeviceVolume");
    [self.rtcEngine setRecordingDevicesVolume:volume];
}


@end
