//
//  MainViewController.m
//  OpenLive
//
//  Created by yangmoumou on 2018/2/8.
//  Copyright © 2018年 yangmoumou. All rights reserved.
//

#import "MainViewController.h"
#import "SettingsViewController.h"
#import "LiveRoomViewController.h"
#import "HLMonitor.h"
#import "HwRtcEngine.h"
static NSString *settingIdentifier = @"mainToSettings";
static NSString *liveIdentifier = @"mainToLive";

@interface MainViewController ()<SettingsVCDelegate, LiveRoomVCDelegate>
@property (weak) IBOutlet NSTextField *roomInputTextFiled;
@property (weak) IBOutlet NSTextField *UserNameTextFiled;

@property (nonatomic, strong) HwRtcEngine *rtcEngine;
@property (nonatomic, assign) HwRtcRole roleType;
@property (assign, nonatomic) BOOL isLastmileProbeTesting;
@property (nonatomic, strong) NSString *address;
@property (nonatomic, assign) CGFloat screenWidth;
@property (nonatomic, assign) CGFloat screenHeight;
@end

@implementation MainViewController


- (void)viewDidLoad {
    [super viewDidLoad];
    _screenWidth = 480;
    _screenHeight = 640;
    NSString *address = [[NSUserDefaults standardUserDefaults] objectForKey:@"address"];
    self.address = address.length>0?address:@"rtc.myhuaweicloud.com";
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [self instanceEngine];
    });
    
    [self insertNotificationCenter];
}

- (void)instanceEngine
{
    _rtcEngine = [HwRtcEngine sharedEngine];
    
    NSString *documentDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *logFilePath = [documentDirectory stringByAppendingPathComponent:@"/HWRtcEngine-log"];
    if (![[NSFileManager defaultManager] fileExistsAtPath:logFilePath]) {
        [[NSFileManager defaultManager] createDirectoryAtPath:logFilePath withIntermediateDirectories:YES attributes:nil error:nil];
    }

    
    int result = [_rtcEngine setLogParam:HwRtcLogLevelDebug logPath:[NSString stringWithFormat:@"%@/",logFilePath] enable:YES];
    NSLog(@"日志地址 ==> %@ ,result = %d",logFilePath,result);
    /*
     多站点环境：
     1、内网:
     
     sdk_center_1_edge_1=100.95.131.16
     sdk_center_2_edge_2=100.94.17.246
     sdk_center_2_edge_3=100.94.16.193
     sdk_center_3_edge_4=100.95.189.41
     
     2、公网：
     public_beta_center_beijing4_edge_beijing4=139.9.118.222
     public_beta_center_shanghai2_edge_shanghai2=121.36.170.200
     public_beta_center_guangzhou_edge_guangzhou=121.37.236.31
     public_beta_center_guangzhou_edge_guiyang=116.63.151.115
     */
    [_rtcEngine init:self.address appId:@"xxx_app_id"];//使用你自己的APPID
    HwRtcVideoEncode *videoEncode = [[HwRtcVideoEncode alloc] init];

    videoEncode.streamFlag = HwRtcEncodeStreamTypeHD|HwRtcEncodeStreamTypeFHD;
    [self.rtcEngine setVideoEncoder:videoEncode];
}

- (void)insertNotificationCenter
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(screenResize) name:NSWindowDidResizeNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidEnterFullScreen:) name:NSWindowDidEnterFullScreenNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidExitFullScreen:) name:NSWindowDidExitFullScreenNotification object:nil];
}

// 窗口拉伸
- (void)screenResize{
    
}

- (void)windowDidEnterFullScreen:(NSNotificationCenter *)notification
{
    _screenWidth = [NSScreen mainScreen].frame.size.width;
    _screenHeight = [NSScreen mainScreen].frame.size.height;

}

- (void)windowDidExitFullScreen:(NSNotificationCenter *)notification
{
    _screenWidth = 480;
    _screenHeight = 640;

}


- (IBAction)doSettingsClicked:(NSButton *)sender {
    [self performSegueWithIdentifier:settingIdentifier sender:nil];
}


- (IBAction)doJoinWitRoleTypeJoiner:(NSButton *)sender {
    self.roleType = HwRtcRoleJoiner;
    [self pushLiveRoomVC];
}

- (IBAction)doJoinWithRoleTypePlayer:(id)sender {
    self.roleType = HwRtcRolePlayer;
    [self pushLiveRoomVC];
}

- (IBAction)doJoinWithRoleTypePulish:(NSButton *)sender {
    self.roleType = HwRtcRolePublisher;
    [self pushLiveRoomVC];
}

- (void)pushLiveRoomVC {
    if (self.roomInputTextFiled.stringValue.length==0 ||
        self.UserNameTextFiled.stringValue.length==0 ||
        self.roomInputTextFiled.stringValue.length>=128 ||
        self.UserNameTextFiled.stringValue.length>=128) {
        [HLMonitor showMessageWithStr:@"请输入正确的roomid和userid"];
        return;
    }
    [self performSegueWithIdentifier:liveIdentifier sender:nil];
}

- (void)prepareForSegue:(NSStoryboardSegue *)segue sender:(id)sender {
    if ([segue.identifier isEqualToString:settingIdentifier]) {
        SettingsViewController *settingVC = segue.destinationController;
        settingVC.rtcEngine = self.rtcEngine;
        settingVC.delegate = self;
        settingVC.address = self.address;
    }else if ([segue.identifier isEqualToString:liveIdentifier]) {
        LiveRoomViewController *liveRoomVC = segue.destinationController;
        liveRoomVC.rtcEngine = self.rtcEngine;
        liveRoomVC.roomName = self.roomInputTextFiled.stringValue;
        liveRoomVC.userName = self.UserNameTextFiled.stringValue;
        liveRoomVC.delegate = self;
        liveRoomVC.roleType = self.roleType;
        liveRoomVC.address = self.address;
        liveRoomVC.screenWidth = self.screenWidth;
        liveRoomVC.screenHeight = self.screenHeight;
    }
}

- (void)settingsVC:(SettingsViewController *)settingsVC address:(NSString *)address{
    settingsVC.view.window.contentViewController = self;
    settingsVC.delegate = nil;
    if (![self.address isEqualToString:address]) {
        self.address = address;
        // 如果appid 和 address 发生修改 需要销毁SDK 重新初始化
        [[HwRtcEngine sharedEngine] destroy];
        [[NSUserDefaults standardUserDefaults] setObject:address forKey:@"address"];
        [self instanceEngine];
    }
}

- (void)liveVCNeedClose:(LiveRoomViewController *)liveVC {
    /// 重新设置底部widow的窗口大小
    _screenWidth = liveVC.screenWidth;
    _screenHeight = liveVC.screenHeight;
    liveVC.view.window.contentViewController = self;
    liveVC.delegate = nil;
    
    self.view.window.restorable = NO;
    [self.view.window setContentSize:NSMakeSize(_screenWidth, _screenHeight)];
}

@end
