
#ifndef __HW_RTC_DEF_H__
#define __HW_RTC_DEF_H__

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif /*  __cpluscplus */
#endif /*  __cpluscplus */


#ifdef WIN32
#define RTC_API __declspec(dllexport)
#else 
#define RTC_API  __attribute__ ((__visibility__ ("default")))  
#endif

#define RTC_SDK_VERSION "1.0.1"

#define RTC_MAX_DOMIAN_LEN    (128)
#define RTC_MAX_IP_LEN        (128)
#define RTC_MAX_APPID_LEN     (128)
#define RTC_MAX_LOG_PATH_LEN  (226)
#define RTC_MAX_USERID_LEN    (128)
#define RTC_MAX_USERNAME_LEN  (128)
#define RTC_MAX_ROOMID_LEN    (128)
#define RTC_MAX_NICK_NAME     (128)
#define RTC_MAX_TOKEN_LEN     (128)
#define RTC_MAX_SIGNATURE_TIMESTAMP_LEN  (128)
#define RTC_MAX_SIGNATURE_METHOD_LEN  (128)
#define RTC_MAX_SIGNATURE_NONCE_LEN  (128)
#define RTC_MAX_ERROR_CODE_LEN (12)
#define RTC_MAX_ERROR_MSG_LEN (256)
#define RTC_MAX_DESCRIPTION_MSG_LEN (256)
#define RTC_MAX_DEVICE_NAME_LEN (256)
#define RTC_MAX_DEVICE_ID (256)

#define RTC_OK  (0)

#if defined(__LP64__) || defined(_WIN64)
typedef long long              RTC_INTPTR_T;
typedef unsigned long long     RTC_UINTPTR_T;
#else
typedef signed int             RTC_INTPTR_T;
typedef unsigned int           RTC_UINTPTR_T;
#endif


typedef enum tagRtcErrorCode {
    RTC_ERR_CODE_CLIENT_SYSTEM_ERROR = 90000001,
    RTC_ERR_CODE_MSG_TOOLARGE,
    RTC_ERR_CODE_MEM_NOT_ENOUGH,
    RTC_ERR_CODE_SYNSEND_MSG_ERR,
    RTC_ERR_CODE_PARAM_ERROR,
    RTC_ERR_CODE_API_CALLED_IN_WRONG_ORDER,
    RTC_ERR_CODE_SETUP_LOCAL_VIEW_FAIL,
    RTC_ERR_CODE_SETUP_REMOTE_VIEW_FAIL,
    RTC_ERR_CODE_GET_SERVERIP_FAIL,
    RTC_ERR_CODE_IP_UNREACHABLE,
    RTC_ERR_CODE_SET_DEVICE_FAIL,
    RTC_ERR_CODE_BUTT    /**/
} RtcErrorCode;


typedef enum tagRtcNotify {
    RTC_NOTIFY_BEGIN = 0,
    RTC_NOTIFY_WARNING,
    RTC_NOTIFY_ERROR, /* RtcErrorNotify*/
    RTC_NOTIFY_JOIN_ROOM_SUCCESS, /* JoinRoomSuccessNotify */
    RTC_NOTIFY_LEAVE_ROOM, /* LeaveRoomNotify */
    RTC_NOTIFY_USER_JOINED, /* UserJoinedNotify */
    RTC_NOTIFY_USER_OFFLINE, /* UserOfflineNotify */
    RTC_NOTIFY_FIRST_REMOTE_VIDEO_DECODED, /* FirstRemoteVideoDecodedNotify */
    RTC_NOTIFY_CONN_STATE_CHANGE, /*RtcConnStateChangeNotify*/
    RTC_NOTIFY_DEVICE_CHANGE, /* RtcDeviceStateChangedNotify */
    RTC_NOTIFY_AUDIO_VOLUME_CHANGE, /* RtcAudioVolumeNotify */
    RTC_NOTIFY_END
}RtcNotify;


typedef enum tagRtcLogLevel {
    RTC_LOG_LEVEL_ERROR = 0,
    RTC_LOG_LEVEL_WARNING,
    RTC_LOG_LEVEL_INFO,
    RTC_LOG_LEVEL_DEBUG
}RtcLogLevel;

typedef enum tagRtcTokenType {
    RTC_TOKEN_APPID_KEY = 0,
    RTC_TOKEN_CERT,
    RTC_TOKEN_THIRDSSO,
    RTC_TOKEN_CRT_THIRDSSO    
}RtcTokenType;

typedef enum tagRtcRole {
    RTC_ROLE_JOINER = 0,
    RTC_ROLE_PUBLISHER,
    RTC_ROLE_PLAYER
}RtcRole;

typedef enum tagRtcMediaType {
    RTC_MEDIA_TYPE_AUDIO = 0,
    RTC_MEDIA_TYPE_VIDEO,
    RTC_MEDIA_TYPE_DATA
}RtcMediaType;

typedef enum tagRtcVideoDisplayMode {
    RTC_VIDEO_DISPLAY_ZOOM = 0,
    RTC_VIDEO_DISPLAY_PAD,
    RTC_VIDEO_DISPLAY_CROP,
    RTC_VIDEO_DISPLAY_ORIG       
}RtcVideoDisplayMode;

typedef enum tagRtcStreamType {
    RTC_STREAM_TYPE_SD,
    RTC_STREAM_TYPE_HD
}RtcStreamType;

typedef enum tagRtcSpeakerModel {
    RTC_SPEAKER_MODEL_EARPIECE,
    RTC_SPEAKER_MODEL_SPEAKER
}RtcSpeakerModel;

typedef enum tagRtcDeviceType{
    RTC_DEVTYPE_AUDIO_PLAYBACK,
    RTC_DEVTYPE_AUDIO_RECORDING,
    RTC_DEVTYPE_VIDEO_CAPTURE
}RtcDeviceType;

typedef enum tagRtcDeviceState{
    RTC_DEVICE_STATE_ACTIVE,
    RTC_DEVICE_STATE_DISABLED,
    RTC_DEVICE_STATE_UNPLUGGED
}RtcDeviceState;

typedef struct tagMaterialInfo {
    char signatureTimeStamp[RTC_MAX_SIGNATURE_TIMESTAMP_LEN + 1];
    char signatureMethod[RTC_MAX_SIGNATURE_METHOD_LEN + 1];
    char signatureNonce[RTC_MAX_SIGNATURE_NONCE_LEN + 1];
}MaterialInfo;

typedef struct tagRtcUserInfo {
    char userId[RTC_MAX_USERID_LEN + 1];
    char userName[RTC_MAX_USERNAME_LEN + 1];
    char token[RTC_MAX_TOKEN_LEN + 1];
    RtcTokenType tokenType;
    RtcRole role;
    MaterialInfo materiaInfo;
}RtcUserInfo;

typedef struct tagRtcJoinRoomInfo {
    RtcUserInfo userInfo;
    RtcMediaType mediaType;
    char roomId[RTC_MAX_ROOMID_LEN + 1];
}RtcJoinRoomInfo;

typedef struct tagRtcVideoLocalView {
    RTC_UINTPTR_T render;                             /* ھ */
    RtcVideoDisplayMode viewMode;
}RtcVideoLocalView;

typedef struct tagRtcVideoRemoteView {
    RTC_UINTPTR_T render;                            /* ھ */
    RtcVideoDisplayMode viewMode;                    /* ʾģʽ */
    RtcStreamType streamType;                        /* ģʽ */
    char userId[RTC_MAX_USERID_LEN + 1];
}RtcVideoRemoteView;

typedef struct tagRtcRotationParam {
    int captureIndex;
    int captureRotation;
    int wndType;
    int displayRotation;
}RtcRotationParam;

typedef struct tagRtcDeviceInfo {
    char deviceName[RTC_MAX_DEVICE_NAME_LEN + 1];
    char deviceId[RTC_MAX_DEVICE_ID + 1];
}RtcDeviceInfo;

typedef struct tagRtcErrorNotify {
    char errorCode[RTC_MAX_ERROR_CODE_LEN + 1];
    char errorMsg[RTC_MAX_ERROR_MSG_LEN + 1];
}RtcErrorNotify;

typedef struct tagJoinRoomSuccessNotify {
    char roomId[RTC_MAX_ROOMID_LEN + 1];
    char userId[RTC_MAX_USERID_LEN + 1];
}JoinRoomSuccessNotify;

typedef struct tagLeaveRoomNotify {
    char roomId[RTC_MAX_ROOMID_LEN + 1];
    char userId[RTC_MAX_USERID_LEN + 1];
}LeaveRoomNotify;

typedef struct tagUserJoinedNotify {
    char roomId[RTC_MAX_ROOMID_LEN + 1];
    char userId[RTC_MAX_USERID_LEN + 1];
    char nickName[RTC_MAX_NICK_NAME + 1];
}UserJoinedNotify;

typedef struct tagUserOfflineNotify {
    char roomId[RTC_MAX_ROOMID_LEN + 1];
    char userId[RTC_MAX_USERID_LEN + 1];
    int reason;
    RTC_UINTPTR_T render;  /* ھ */
}UserOfflineNotify;

typedef struct tagFirstRemoteVideoDecodedNotify {
    char userId[RTC_MAX_USERID_LEN + 1];
    int width;
    int height;
}FirstRemoteVideoDecodedNotify;

// ״̬
typedef enum tagRtcConnStateTypes {
    RTC_CONN_DISCONNECTED = 0, // ӶϿ
    RTC_CONN_CONNECTING,       // 
    RTC_CONN_CONNECTED,        // ӳɹ
    RTC_CONN_RECONNECTING,     // ½
    RTC_CONN_FAILED            // ʧ
} RtcConnStateType;

typedef enum tagRtcConnChangeReason {
    RTC_CONN_CHANGED_CONNECTING = 0,           // 
    RTC_CONN_CHANGED_JOIN_SUCCESS,             // 뷿ɹ
    RTC_CONN_CHANGED_RECONNECTING,             // 
    RTC_CONN_CHANGED_RECONNECT_SUCCESS,        // ɹ
    RTC_CONN_CHANGED_JOIN_FAILED,              // 뷿ʧ
    RTC_CONN_CHANGED_RECONNCET_FAILED,         // ʧ
    RTC_CONN_CHANGED_RECONNCET_BANNED,         // ʧ403
    RTC_CONN_CHANGED_INTERRUPTED,              // ж
    RTC_CONN_CHANGED_BANNED_BY_SERVER,         // Ȩʧ403
    RTC_CONN_CHANGED_KEEP_ALIVE_TIMEOUT,       // ʱ
    RTC_CONN_CHANGED_LEAVE_ROOM,               // 뿪
    RTC_CONN_CHANGED_JOIN_SERVER_ERROR,        // 쳣
    RTC_CONN_CHANGED_SFU_BREAKDOWN             // sfu
} RtcConnChangeReason;

typedef struct tagConnStateChangeNotify {
    RtcConnStateType state;
    RtcConnChangeReason reason;
    char description[RTC_MAX_DESCRIPTION_MSG_LEN + 1];
}ConnStateChangeNotify;

typedef struct tagRtcDeviceStateChangedNotify {
    char deviceId[RTC_MAX_DEVICE_ID + 1];
    RtcDeviceType deviceType;
    RtcDeviceState deviceState;
} RtcDeviceStateChangedNotify;

typedef struct tagRtcAudioVolumeNotify {
    RtcDeviceType deviceType;
    unsigned int  volume;
} RtcAudioVolumeNotify;

typedef void (*RtcNotifyFunc)(unsigned int msgId, unsigned int param1, unsigned int param2, void *data);

#ifdef __cplusplus
#if __cplusplus
}
#endif /*  __cpluscplus */
#endif /*  __cpluscplus */


#endif /*__HW_RTC_DEF_H__*/

